//---------------------------------------------------------------------
//  Algorithmic Conjurings @ http://www.coyotegulch.com
//
//  Messiggy - A database of Messier objects, using the Itzam engine
//
//  callbacks.c
//
//---------------------------------------------------------------------
//
//  Copyright 2004,2005 Scott Robert Ladd
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//      Free Software Foundation, Inc.
//      59 Temple Place - Suite 330
//      Boston, MA 02111-1307, USA.
//
//-----------------------------------------------------------------------
//
//  For more information on this software package, please visit
//  Scott's web site, Coyote Gulch Productions, at:
//
//      http://www.coyotegulch.com
//
//-----------------------------------------------------------------------

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk-pixbuf/gdk-pixdata.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "itzam-core/itzam.h"

#include <math.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>


/***********************************************************************
    Main window
************************************************************************/


void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


static const char * ABOUT_MESSIGGY =
    "<span foreground=\"blue\" size=\"x-large\">Messiggy %s</span>\n\n"
    "A simple database of Messier objects.\n\n"
    "<b>http://messiggy.coyotegulch.com</b>\n\n"
    "Copyright 2005 Scott Robert Ladd.\n"
    "All rights reserved.\n";

static const char * ABOUT_ITZAM =
    "Messiggy uses the\n"
    "<span foreground=\"#008000\" size=\"x-large\">Itzam</span>\n"
    "embedded database engine.\n\n"
    "<b>http://itzam.coyotegulch.com</b>";


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    // create the window
    GtkWidget *about_window;
    about_window = create_about_dialog ();
    gtk_widget_show (about_window);
    
    // generate about with proper version
    char vertext[1024];
    snprintf(vertext,1024,ABOUT_MESSIGGY,VERSION);

    // set text describing messiggy
    GtkWidget * label = lookup_widget (GTK_WIDGET (about_window),"about_messiggy_label");
    gtk_label_set_use_markup(GTK_LABEL(label), true);
    gtk_label_set_markup(GTK_LABEL(label), vertext);

    // set text describing itzam
    label = lookup_widget (GTK_WIDGET (about_window),"about_itzam_label");
    gtk_label_set_use_markup(GTK_LABEL(label), true);
    gtk_label_set_markup(GTK_LABEL(label), ABOUT_ITZAM);
}

void
on_main_window_realize                 (GtkWidget       *widget,
                                        gpointer         user_data)
{

}

gboolean
on_main_window_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_main_quit();
  return FALSE;
}


gboolean
on_main_window_destroy_event           (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_main_quit();
  return FALSE;
}


/***********************************************************************
    Events for Image Map
************************************************************************/

static const int OBJECT_XDIM = 50;
static const int OBJECT_YDIM = 38;
static const int OBJECT_COLS = 10;
static const int MAP_XDIM    = 500;
static const int MAP_YDIM    = 418;

static int get_object_index(int x, int y)
{
    return (y / OBJECT_YDIM) * OBJECT_COLS + x / OBJECT_XDIM + 1;
}

static GdkPixbuf * map_dark = NULL;
static GdkPixbuf * map_lite = NULL;

static const char * MAP_DARK_NAME = PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps/messmapdark.png";
static const char * MAP_LITE_NAME = PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps/messmaplite.png";

void draw_map_base(GtkWidget * messier_map)
{
    // load image if we need to
    if (map_dark == NULL)
    {
        map_dark = gdk_pixbuf_new_from_file (MAP_DARK_NAME, NULL);

        if (map_dark == NULL)
            g_printf("unable to open %s\n",MAP_DARK_NAME);
    }

    // draw image
    gdk_draw_pixbuf(messier_map->window,
                    messier_map->style->black_gc,
                    map_dark, 0, 0,
                    0, 0, MAP_XDIM, MAP_YDIM,
                    GDK_RGB_DITHER_NORMAL, 0, 0);
}

static void draw_map_hilite(GtkWidget * messier_map, int x, int y)
{
    // load image if we need to
    if (map_lite == NULL)
    {
        map_lite = gdk_pixbuf_new_from_file (MAP_LITE_NAME, NULL);

        if (map_lite == NULL)
            g_printf("unable to open %s\n",MAP_LITE_NAME);
    }

    // calculate stating position of highlighted image
    int origin_x = (x / OBJECT_XDIM) * OBJECT_XDIM;
    int origin_y = (y / OBJECT_YDIM) * OBJECT_YDIM;

    // draw image
    gdk_draw_pixbuf(messier_map->window,
                    messier_map->style->black_gc,
                    map_lite, origin_x, origin_y,
                    origin_x, origin_y,
                    OBJECT_XDIM, OBJECT_YDIM,
                    GDK_RGB_DITHER_NORMAL, 0, 0);
}

gboolean
on_messier_map_motion_notify_event     (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
    // find index into 110-object messier list, based on click position
    int index = get_object_index(event->x,event->y);

    // generate text for status line
    char key[6];
    sprintf(key,"M%d",index);

    // draw window
    draw_map_base(widget);
    draw_map_hilite(widget,event->x,event->y);

    // update status
    GtkWidget * statusbar = lookup_widget (GTK_WIDGET (widget), "statusbar");
    gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 1);
    gtk_statusbar_push (GTK_STATUSBAR (statusbar), 1, key);

    return FALSE;
}

struct messrec
{
    uint16_t messier_no;
    char common_name[40];
    char type[24];
    char constellation[20];
    uint16_t ngc_ic_cat_no;
    float magnitude;
    char size[12];
    float ra;
    float dec;
    uint32_t image_size;
    itzam_ref image_ref;
    char image_credit[64];
    itzam_ref annot_ref;
};

static const size_t CREDIT_LEN = 64;

int string_comparator(const void * key1, const void * key2)
{
    return strcmp((const char *)key1,(const char *)key2);
}

static const char * RAW_NAME = PACKAGE_DATA_DIR "/" PACKAGE "/data/messier.txt";
static const char * CREDITS_NAME = PACKAGE_DATA_DIR "/" PACKAGE "/data/credits.txt";

static const char * WORK_DIR = ".messiggy";
static const char * DB_NAME = "messiggy.db";

static const char * PIX_PATH = PACKAGE_DATA_DIR "/" PACKAGE "/pixmaps/m%03d.jpg";

static const char * MESS_FORM =
        "<big><b>%s</b></big>\n"
        "<span foreground=\"blue\" weight=\"bold\">%s</span>\n\n"
        "%s in the\nconstellation %s.\n\n"
        "magnitude: %3.1f\n\n"
        "size: %s\"\n\n"
        "RA\t%s\nDec\t%s\n";

static const char * RA_FORM =
        "%2.0f<span size=\"x-small\" rise=\"5000\">h</span> "
        "%4.1f<span size=\"x-small\" rise=\"5000\">m</span>";

static const char * DEC_FORM =
        "%2.0f<span size=\"x-small\" rise=\"6000\">o</span> "
        "%4.1f\'";

static const char * CREDIT_FORM =
        "<span size=\"small\">credit: <i>%s</i></span>";

// prototypes for default internal (crash) error handling
void db_error(const char * function_name, itzam_error error)
{
    GtkWidget * dialog = gtk_message_dialog_new(NULL,
                                    GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_ERROR,
                                    GTK_BUTTONS_CLOSE,
                                    "Error",
                                    function_name);

    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
}

static void show_message(const char * message, GtkMessageType type)
{
    GtkWidget * dialog = gtk_message_dialog_new(NULL,
                                    GTK_DIALOG_DESTROY_WITH_PARENT,
                                    type,
                                    GTK_BUTTONS_CLOSE,
                                    message);

    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
}

static char * get_db_name()
{
    static char db_name[1024] = { 0 };

    if (db_name[0] == 0)
    {
        // find home directory
        char * home_dir = getenv("HOME");

        // create a directory
        snprintf(db_name,1024,"%s/%s",home_dir,WORK_DIR);
        mkdir(db_name,0777);

        // now generate the file name
        snprintf(db_name,1024,"%s/%s/%s",home_dir,WORK_DIR,DB_NAME);
    }

    return db_name;
}

static void create_db()
{
    // create dialog box
    //GtkWidget *dialog;
    //dialog = create_db_build_dialog ();
    //gtk_widget_show (dialog);

    // allocate buffer for incoming lines
    static const int BUF_LEN = 128;
    char * buffer = (char *)malloc(BUF_LEN);

    if (buffer == NULL)
        exit(1);

    // create database
    itzam_btree database;

    // TODO: database isn't initialized yet!
    //itzam_btree_set_error_handler(&database,db_error);

    if (ITZAM_OKAY != itzam_btree_create(&database,get_db_name(),string_comparator))
    {
        show_message("Unable to create master database",GTK_MESSAGE_WARNING);
        return;
    }

    // open delimited data file
    FILE * infile = fopen(RAW_NAME,"r");
    FILE * crfile = fopen(CREDITS_NAME,"r");

    struct messrec record;
    int count = 0;
    struct stat image_stats;
    void * image_blob;

    if (infile != NULL)
    {
        // read lines until there aren't any more
        while (NULL != fgets(buffer,BUF_LEN,infile))
        {
            // get a credit line for the image
            memset(record.image_credit,0,CREDIT_LEN);
            fgets(record.image_credit,CREDIT_LEN,crfile);

            // increment count (which is also Messier number)
            ++count;

            // generate name for icon/small image
            char image_name[1024];
            snprintf(image_name,1024,PIX_PATH,count);

            // stat file to find length
            if (0 == stat(image_name,&image_stats))
            {
                // read image from file
                GdkPixbuf * image_buf = gdk_pixbuf_new_from_file(image_name,NULL);

                // get image data from buffer
                GdkPixdata pixdata;
                gpointer free_me = gdk_pixdata_from_pixbuf (&pixdata, image_buf, TRUE);

                // serialize picdata to something we can store
                guint image_raw_len;
                image_blob = (void *)gdk_pixdata_serialize(&pixdata,&image_raw_len);
                record.image_size = image_raw_len;

                // clean up
                g_free(free_me);
            }
            else
            {
                record.image_size = 0;
                image_blob = NULL;
            }

            // start tokenizing
            char * token = strtok(buffer,"|");
            int field_index = 0;

            while (token != NULL)
            {
                // create a record
                if (field_index < 9)
                {
                    switch (field_index)
                    {
                        case 0:
                            record.messier_no = (uint16_t)count;
                            break;

                        case 1:
                            memset(record.common_name,40,0);
                            strcpy(record.common_name,token);
                            break;

                        case 2:
                            memset(record.type,24,0);
                            strcpy(record.type,token);
                            break;

                        case 3:
                            memset(record.constellation,20,0);
                            strcpy(record.constellation,token);
                            break;

                        case 4:
                            record.ngc_ic_cat_no = (uint16_t)atoi(token);
                            break;

                        case 5:
                            record.magnitude = atof(token);
                            break;

                        case 6:
                            memset(record.size,12,0);
                            strcpy(record.size,token);
                            break;

                        case 7:
                        {
                            char * p = strstr(token,"h");
                            *p = 0;
                            record.ra = atof(token);

                            ++p;
                            record.ra += atof(p) / 60.0F;
                        }
                        break;

                        case 8:
                        {
                            char * p = strstr(token," ");
                            *p = 0;
                            record.dec = atof(token);

                            ++p;
                            record.dec += atof(p) / 60.0F;
                        }
                        break;

                    }

                }

                // next token
                token = strtok(NULL,"|");
                ++field_index;
            }

            // generate record key
            char key[6];
            sprintf(key,"M%03d",count);

            // write image blob to file

            if (image_blob != NULL)
            {
                record.image_ref = itzam_datafile_write(database.m_datafile,
                                                        image_blob,
                                                        record.image_size,
                                                        ITZAM_NULL_REF);
                

            }
            else
                record.image_ref = ITZAM_NULL_REF;

            // new records do not have annotations yet
            record.annot_ref = ITZAM_NULL_REF;

            // write record to database
            itzam_btree_insert_rec(&database,
                                   key, strlen(key) + 1,
                                   &record, sizeof(record));
        }

        // close file
        fclose(infile);

        // close database
        itzam_btree_close(&database);

        // destroy the dialog box
        // gtk_widget_destroy (dialog);
    }
    else
        fprintf(stderr,"Unable to open input file\n");

    // free pointer
    free(buffer);
}

void
on_build_database1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    create_db();
}

struct tree_item
{
    const gchar * title;
    const gchar * date;
    const gchar * info;
    const gchar * comment;
    struct tree_item * children;
};

enum
{
    TITLE_COLUMN = 0,
    DATE_COLUMN,
    INFO_COLUMN,
    COMMENT_COLUMN
};

// dummy data for testing
static struct tree_item images[] =
{
    { "Picture1","17-Oct-2003","640x480","Taken with ST-8", NULL },
    { "Picture2","10-Jun-2003","640x480","Taken with ST-8", NULL },
    { "Picture3","01-May-2004","640x480","Taken with ST-8", NULL },
    { "Picture4","30-Mar-2005","640x480","Taken with ST-8", NULL },
    { NULL }
};

static struct tree_item notes[] =
{
    { "Note1","17-Oct-2003","640x480","Observations", NULL },
    { "Note2","10-Jun-2003","640x480","Observations", NULL },
    { "Note3","01-May-2004","640x480","Observations", NULL },
    { NULL }
};

static struct tree_item links[] =
{
    { "NOAO","17-Oct-2003","http://www.noao.edu","NOAO", NULL },
    { "NASA","10-Jun-2003","http://www.nasa.gov","NASA", NULL },
    { "JPL","01-May-2004","http://www.jpl.nasa.gov","Jet Propulsion Lab", NULL },
    { NULL }
};

static struct tree_item categories[] =
{
    { "Images", NULL, NULL, NULL, images },
    { "Notes",  NULL, NULL, NULL, notes },
    { "Links",  NULL, NULL, NULL, links },
    { NULL }
};

static GtkTreeModel *
create_model (void)
{
    GtkTreeStore * model;
    GtkTreeIter    iter;
    struct tree_item * category = categories;

    // create tree store
    model = gtk_tree_store_new (4,
                                G_TYPE_STRING,
                                G_TYPE_STRING,
                                G_TYPE_STRING,
                                G_TYPE_STRING);

    // add data to the tree store
    while (category->title != NULL)
    {
        struct tree_item * child = category->children;

        gtk_tree_store_append (model, &iter, NULL);

        gtk_tree_store_set (model, &iter,
                            TITLE_COLUMN,   category->title,
                            DATE_COLUMN,    NULL,
                            INFO_COLUMN,    NULL,
                            COMMENT_COLUMN, NULL,
                            -1);

        // add children
        while (child->title != NULL)
        {
            GtkTreeIter child_iter;

            gtk_tree_store_append (model, &child_iter, &iter);

            gtk_tree_store_set (model, &child_iter,
                                TITLE_COLUMN,   child->title,
                                DATE_COLUMN,    child->date,
                                INFO_COLUMN,    child->info,
                                COMMENT_COLUMN, child->comment,
                                -1);

            child++;
        }

        category++;
    }

    return GTK_TREE_MODEL (model);
}

static void
add_columns (GtkTreeView *treeview)
{
    GtkTreeViewColumn * column;
    GtkTreeModel *      model = gtk_tree_view_get_model (treeview);

    // category & title column
    GtkCellRenderer * renderer = gtk_cell_renderer_text_new ();
    g_object_set (renderer, "xalign", 0.0, NULL);

    gint col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                                                   -1, "Title",
                                                                   renderer,
                                                                   "text", TITLE_COLUMN,
                                                                   NULL);

    column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), col_offset - 1);
    gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);

    // date column
    renderer = gtk_cell_renderer_text_new ();
    g_object_set (renderer, "xalign", 0.0, NULL);

    col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                                              -1, "Date",
                                                              renderer,
                                                              "text",DATE_COLUMN,
                                                              NULL);

    column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), col_offset - 1);
    gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);

    // info column
    renderer = gtk_cell_renderer_text_new ();
    g_object_set (renderer, "xalign", 0.0, NULL);

    col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                                              -1, "Information",
                                                              renderer,
                                                              "text",INFO_COLUMN,
                                                              NULL);

    column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), col_offset - 1);
    gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);

    // comment column
    renderer = gtk_cell_renderer_text_new ();
    g_object_set (renderer, "xalign", 0.0, NULL);

    col_offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                                              -1, "Comment",
                                                              renderer,
                                                              "text",COMMENT_COLUMN,
                                                              NULL);

    column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), col_offset - 1);
    gtk_tree_view_column_set_clickable (GTK_TREE_VIEW_COLUMN (column), TRUE);
    
    // set the selection mode
    gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview)),
                                 GTK_SELECTION_MULTIPLE);

    // expand all categories
    g_signal_connect (treeview, "realize", G_CALLBACK (gtk_tree_view_expand_all), NULL);
}

static GdkPixbuf * get_record(int index, struct messrec * record)
{
    // generate a database key
    char key[6];
    snprintf(key,6,"M%03d",index);

    // open database, read record
    itzam_btree database;

    if (ITZAM_OKAY != itzam_btree_open(&database,get_db_name(),string_comparator,true))
    {
        // try creating the db
        create_db();

        // now open it again
        if (ITZAM_OKAY != itzam_btree_open(&database,get_db_name(),string_comparator,true))
        {
            show_message("Unable to open master itzam database",GTK_MESSAGE_ERROR);
            exit(1);
        }
    }

    // read record
    itzam_btree_read_rec(&database,key,record,sizeof(struct messrec));

    // read image from file
    guint8 * image_raw = (guint8 *)malloc(record->image_size);
    itzam_datafile_seek(database.m_datafile,record->image_ref);
    itzam_datafile_read(database.m_datafile,image_raw,record->image_size);

    // deserialize the data
    GdkPixdata image_data;
    gdk_pixdata_deserialize(&image_data,(guint)record->image_size,image_raw,NULL);
    GdkPixbuf * image_buf = gdk_pixbuf_from_pixdata(&image_data,TRUE,NULL);

    // free memory
    free(image_raw);

    // close database
    itzam_btree_close(&database);

    // done
    return image_buf;
}

gboolean
on_messier_map_button_press_event      (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
    // find index into 110-object messier list, based on click position
    int index = get_object_index(event->x,event->y);

    // generate "M"-name for object
    char name[6];
    snprintf(name,6,"M%d",index);

    // update status bar
    GtkWidget * statusbar = lookup_widget (GTK_WIDGET (widget), "statusbar");
    gtk_statusbar_pop (GTK_STATUSBAR (statusbar), 1);
    gtk_statusbar_push (GTK_STATUSBAR (statusbar), 1, name);

    // read record
    struct messrec record;
    GdkPixbuf * image_buf = get_record(index,&record);

    // generate window title
    char title[128];
    snprintf(title,128,"Messiggy: %s",name);

    // generate name for icon/small image
    char icon_name[1024];
    snprintf(icon_name,1024,PIX_PATH,index);

    // generate RA text
    char ra_text[256];
    float ra_int;
    float ra_frac = fabsf(modff(record.ra,&ra_int)) * 60.0F;
    snprintf(ra_text,256,RA_FORM, ra_int, ra_frac);

    // generate DEC text
    char dec_text[256];
    float dec_int;
    float dec_fdecc = fabsf(modff(record.dec,&dec_int)) * 60.0F;
    snprintf(dec_text,256,DEC_FORM,dec_int, dec_fdecc);

    // generate descriptive text
    char mtext[1024];
    snprintf(mtext,1024,MESS_FORM,
             name, record.common_name,
             record.type, record.constellation,
             record.magnitude, record.size,
             ra_text, dec_text);

    // generate credit text
    char crtext[128];
    snprintf(crtext,128,CREDIT_FORM,record.image_credit);

    // create and display window
    GtkWidget * messobj_window = create_object_window ();
    gtk_window_set_title(GTK_WINDOW(messobj_window),title);
    gtk_window_set_icon_from_file(GTK_WINDOW(messobj_window),icon_name,NULL);
    gtk_widget_show (messobj_window);

    // define tree model and contents
    GtkTreeModel * model = create_model();
    GtkWidget * treeview = lookup_widget (GTK_WIDGET (messobj_window),"object_annotations");
    gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), model);
    g_object_unref(model);
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);
    add_columns (GTK_TREE_VIEW (treeview));

    // set text describing the object
    GtkWidget * label = lookup_widget (GTK_WIDGET (messobj_window),"object_description");
    gtk_label_set_use_markup(GTK_LABEL(label), true);
    gtk_label_set_markup(GTK_LABEL(label), mtext);

    // set text describing the object
    GtkWidget * credit = lookup_widget (GTK_WIDGET (messobj_window),"object_image_credit");
    gtk_label_set_use_markup(GTK_LABEL(credit), true);
    gtk_label_set_markup(GTK_LABEL(credit), crtext);

    // display the image
    GtkWidget * icon = lookup_widget (GTK_WIDGET (messobj_window),"object_icon");
    gtk_image_set_from_pixbuf(GTK_IMAGE(icon), image_buf);

    // done
    return FALSE;
}

gboolean
on_messier_map_leave_notify_event      (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
    draw_map_base(widget);
    return FALSE;
}


gboolean
on_messier_map_expose_event            (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
    draw_map_base(widget);
    return FALSE;
}

/***********************************************************************
    "Not Implemented" Dialog box
************************************************************************/

void
on_not_impl_close_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


/***********************************************************************
    "About" Dialog box
************************************************************************/

void
on_about_close_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}

/***********************************************************************
    "Building DB" Dialog box
************************************************************************/

void
on_db_build_close_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}

/***********************************************************************
    Messier Object Display Window
************************************************************************/

void
on_object_close_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}

static const size_t MAX_TITLE   =  64;
static const size_t MAX_INFO    =  32;
static const size_t MAX_COMMENT = 256;

static const uint32_t TAG_IMAGE = 0;
static const uint32_t TAG_TEXT  = 1;
static const uint32_t TAG_LINK  = 2;

struct annotation
{
    itzam_ref prev_annot;   // link to previous annotation in list
    itzam_ref next_annot;   // link to next annotation in list
    itzam_ref object;       // actual object data
    uint32_t  date;         // a standard 32-bit unix time
    uint32_t  tag;          // type of annotation
    char      title[64];    // title for this annotation
    char      info[32];     // size for image, length for files, constant for a link
    char      comment[256]; // user-defined comment about this annotation
};


/***********************************************************************
    Annotation actions
************************************************************************/

void
on_object_annot_add_image_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget * dialog = create_add_image_dialog();
  gtk_widget_show (dialog);

}

void
on_object_annot_add_note_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget * dialog = create_add_note_dialog();
  gtk_widget_show (dialog);
}


void
on_object_annot_add_link_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget * dialog = create_add_link_dialog();
  gtk_widget_show (dialog);

}


void
on_object_annot_remove_button_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_object_annot_open_button_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{

}


/***********************************************************************
    Add a Note dialog
************************************************************************/

void
on_add_note_add_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_add_note_cancel_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


/***********************************************************************
    Add an Image dialog
************************************************************************/

void
on_add_image_browse_button_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_add_image_add_button_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_add_image_cancel_button_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}

/***********************************************************************
    Add a Note dialog
************************************************************************/


void
on_add_link_test_button_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_add_link_add_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_add_link_cancel_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
    gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (button)));
}


/***********************************************************************
    Show a Note dialog
************************************************************************/

void
on_show_note_export_button_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_show_note_cancel_button_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{

}

